#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File              : generate.py
# License           : MIT license <Check LICENSE>
# Author            : Anderson Ignacio da Silva (aignacio) <anderson@aignacio.com>
# Date              : 20.02.2023
# Last Modified Date: 11.06.2023
import logging
import os
import sys
import shutil
from tabulate import tabulate
from jinja2 import Environment, FileSystemLoader

try:
    from ipsocgen.common.constants import options,base_module
    from ipsocgen.common.modules import *
except ImportError:
    from common.constants import options,base_module
    from common.modules import *

def mpsoc_gen(mpsoc, mpsoc_name, desc, output, mmap):
    mpsoc_hdl_obj = []

    # Get all SoC IOs
    socs_list = []
    n_of_tiles = 0
    for tile_id, tile_dic in mpsoc['tiles'].items():
        n_of_tiles = tile_id
        socs_list.append(soc_gen(tile_dic,
                                 'tile_'+str(tile_id),
                                 'MPSoC tile no '+str(tile_id),
                                 output,
                                 mmap, True if tile_id == 0 else False))

    # Clocks and resets
    mpsoc_cfg = {}
    mpsoc_cfg['clk'] = mpsoc['clk']['clk_int']
    mpsoc_cfg['rst'] = mpsoc['rst']['rst_int']
    mpsoc_cfg['txn_id'] = 8

    mpsoc_hdl_obj.append(Clock(mpsoc))
    mpsoc_hdl_obj.append(Reset(mpsoc))
    io, in_s, out_s = _soc_io(mpsoc_hdl_obj)

    # RTL object generation
    # Generate module header with its defines
    global_rtl = ModuleHeader('mpsoc',
                              mpsoc_name,
                              desc,
                              io).get_hdl()

    # Signals to be first declared
    # Extract the signals and add to the RTL
    for obj in mpsoc_hdl_obj:
        for ip in obj.get_signals():
            global_rtl += ip
            global_rtl += '\n'

    global_rtl += '  s_axi_mosi_t [`NUM_TILES-1:0] slaves_axi_mosi;\n'
    global_rtl += '  s_axi_mosi_t [`NUM_TILES-1:0] ravenoc_mosi;\n'
    global_rtl += '  s_axi_miso_t [`NUM_TILES-1:0] slaves_axi_miso;\n'
    global_rtl += '  s_axi_miso_t [`NUM_TILES-1:0] ravenoc_miso;\n'
    global_rtl += '  s_irq_ni_t   [`NUM_TILES-1:0] irqs_ravenoc;\n'
    global_rtl += '  logic '+mpsoc_cfg['rst']+';\n\n'

    # Add the RTL modules of Clock / Reset
    for obj in mpsoc_hdl_obj:
        global_rtl += obj.get_hdl()

    # Instantiate each SoC looking for its clock/reset inputs
    module_info = {}
    module_info['io_clk_mpsoc'] = mpsoc_cfg['clk']
    module_info['io_rst_mpsoc'] = mpsoc_cfg['rst']

    for index, soc in enumerate(socs_list):
        module_info['name'] = soc['name']
        module_info['io_noc_miso'] = 'slaves_axi_mosi['+str(index)+']'
        module_info['io_noc_mosi'] = 'slaves_axi_miso['+str(index)+']'
        module_info['io_noc_irq']  = 'irqs_ravenoc['+str(index)+']'
        module_info['io_clk'] = soc['clk_ext']
        module_info['io_rst'] = soc['rst_ext']
        global_rtl += ModuleInstance(module_info).get_hdl()

    global_rtl += RaveNoC(mpsoc['noc'], mpsoc_cfg, 0, 'instance').get_hdl()
    global_rtl += '\nendmodule'

    _gen_design_files(mpsoc_name, global_rtl, mpsoc_hdl_obj, output)

    ##### DEFINES #####
    # Create the configuration file
    # For MPSoC, use default default definitions for bus+dma
    # TODO: update to use parameters in all IPs instead of macros

    defines = ModuleDefines(mpsoc_name,
                            'mpsoc',
                            'Verilog defines configuration file',
                            _gen_defines_mpsoc(mpsoc['noc'], socs_list[0]['noc_base_addr']))
    _gen_design_files(mpsoc_name+'_defines', defines.get_hdl(), [defines], output)


def soc_gen(soc, soc_name, desc, output, mmap, gen_defines):
    logging.debug('Starting to generate files for SoC - '+soc_name)

    mst = soc['masters']
    slv = soc['slaves']

    soc_cfg = {}
    soc_cfg['clk']     = soc['clk']['clk_int']
    soc_cfg['rst']     = soc['rst']['rst_int']
    soc_cfg['clk_ext'] = soc['clk']['io_in_clk']
    soc_cfg['rst_ext'] = soc['rst']['io_in_rst']
    soc_cfg['txn_id']  = soc['txn_id_width']

    pll = {}
    dma_info = {}
    hdl_obj = []

    # Clocks and resets first
    hdl_obj.append(Clock(soc))
    hdl_obj.append(Reset(soc))

    # Crossbar
    bus = Axi4Bus(soc, mmap)
    hdl_obj.append(bus)

    # Masters
    for master in _soc_master(bus, soc, soc_cfg, dma_info):
        hdl_obj.append(master)

    # Slaves
    for slave in _soc_slaves(bus, soc, soc_cfg, dma_info):
        hdl_obj.append(slave)

    # RTL object generation
    global_rtl = ''

    # Module Header
    io, in_s, out_s = _soc_io(hdl_obj)
    global_rtl += ModuleHeader('soc', soc_name, desc, io).get_hdl()

    # Signals to be first declared
    for obj in hdl_obj:
        for ip in obj.get_signals():
            global_rtl += ip
            global_rtl += '\n'
    global_rtl += '\n'

    # Get the RTL modules
    for obj in hdl_obj:
        global_rtl += obj.get_hdl()

    global_rtl += '\nendmodule'

    logging.debug('Generating RTL for SoC - '+soc_name)
    _gen_design_files(soc_name, global_rtl, hdl_obj, output)
    logging.debug('Generating header C files for SoC - '+soc_name)
    _gen_header_files(soc_name, bus, hdl_obj, output)

    soc_data = {}
    soc_data['name'] = soc_name
    soc_data['input'] = in_s
    soc_data['output'] = out_s
    soc_data['clk_ext'] = soc['clk']['io_in_clk']
    soc_data['rst_ext'] = soc['rst']['io_in_rst']
    soc_data['noc_base_addr'] = bus.get_noc_addr()

    ##### DEFINES #####
    # Create the configuration file
    defines_list = []

    if gen_defines == True:
        # Masters
        for master_or_slaves in hdl_obj:
            defines_list.append(master_or_slaves.get_defines())
        defines = ModuleDefines(soc_name,
                               'soc',
                               'Verilog defines configuration file',
                               defines_list)
        _gen_design_files(soc_name+'_defines', defines.get_hdl(), [defines], output)

    logging.info('All files sucessfully generated for SoC - '+soc_name)
    return soc_data

def _gen_design_files(soc_name, global_rtl, hdl_obj, out_dir):
    types = set()
    # Creates a set without repeated acc types
    for obj in hdl_obj:
        types.add(obj.get_acc_type())

    out_rtl = os.path.join(out_dir,'rtl')

    if not os.path.exists(out_dir):
        os.mkdir(out_dir)
        logging.debug('Creating output directory: '+str(out_dir))
    if not os.path.exists(out_rtl):
        os.mkdir(out_rtl)
        logging.debug('Creating RTL directory: '+str(out_rtl))

    rtl_out = os.path.join(out_rtl,soc_name+'.sv')
    soc_rtl = open(rtl_out, "w")
    soc_rtl.write(global_rtl)
    soc_rtl.close()

def _gen_header_files(soc_name, bus, hdl_obj, out_dir):
    types = set()
    # Creates a set without repeated acc types
    for obj in hdl_obj:
        types.add(obj.get_acc_type())

    out_sw = os.path.join(out_dir,'sw')

    if not os.path.exists(out_dir):
        os.mkdir(out_dir)
        logging.debug('Creating output directory: '+str(out_dir))
    if not os.path.exists(out_sw):
        os.mkdir(out_sw)
        logging.debug('Creating SW directory: '+str(out_sw))

    header_out = os.path.join(out_sw, soc_name+'.h')
    header_file = open(header_out, "w")
    header_file.write('#ifndef '+soc_name.upper()+'_H')
    header_file.write('\n#define '+soc_name.upper()+'_H')
    header_file.write('\n\n')
    header_file.write('// AUTO-GENERATED header file through IPSoCGen')
    header_file.write('\n// '+datetime.now().strftime('%d/%m/%Y %H:%M:%S'))
    header_file.write('\n\n')
    header_file.write(tabulate(bus.table_masters_c, tablefmt="plain"))
    header_file.write('\n\n')
    header_file.write(tabulate(bus.table_slaves_c, tablefmt="plain"))
    header_file.write('\n\n')

    table = []
    for slave in range(len(bus.description)):
        slave_info = _fmt_base_addr(bus.description[slave], bus.base_addr_h[slave])
        table.append(['#define', slave_info['desc'], '0x'+slave_info['baddr']])
    header_file.write(tabulate(table, ['//', 'SLAVE', 'BASE ADDRESS'], tablefmt='plain'))
    header_file.write('\n\n')

    table = []
    for slave in range(len(bus.description)):
        slave_info_mem_size = _fmt_mem_size(bus.description[slave], bus.mem_size[slave])
        #table.append(['#define', slave_info_mem_size['desc'], slave_info_mem_size['mem'], slave_info_mem_size['mem_kib']])
        table.append(['#define', slave_info_mem_size['desc'], slave_info_mem_size['mem']])
    #header_file.write(tabulate(table, ['//', 'SLAVE', 'MEM SIZE BYTES','SIZE KiB'], tablefmt='plain'))
    header_file.write(tabulate(table, ['//', 'SLAVE', 'MEM SIZE BYTES'], tablefmt='plain'))
    header_file.write('\n')

    header_file.write('\n#endif')
    header_file.close()

def _fmt_base_addr(description, base_address):
    info  = {}
    desc  = str(description).replace(' ','_')+'_BASE_ADDR'
    info['desc']  = desc.upper()
    info['baddr'] = str(base_address)
    return info

def _fmt_mem_size(description, mem_size):
    info  = {}
    desc  = str(description).replace(' ','_')+'_SIZE'
    info['desc']  = desc.upper()
    info['mem'] = str(mem_size*1024)
    info['mem_kib'] = '// '+str(mem_size)+'KiB'
    return info

def _soc_slaves(bus, soc, soc_cfg, dma_info):
    hdl_obj = []
    mmap = bus.get_mmap()
    eth_info = {}
    eth_info['if'] = 0

    for slave, slave_desc in soc['slaves'].items():
        if slave_desc['type'] == 'ram_mem':
            iram = Axi4MemRAM(slave_desc, soc_cfg, slave)
            hdl_obj.append(iram)
        elif slave_desc['type'] == 'rom_mem':
            irom = Axi4MemROM(slave_desc, soc_cfg, slave)
            hdl_obj.append(irom)
        elif slave_desc['type'] == 'acc_dma':
            master = dma_info[slave_desc['name']]
            dma = Axi4DMA(slave_desc, soc_cfg, slave, master)
            hdl_obj.append(dma)
        elif slave_desc['type'] == 'acc_uart':
            uart = Axi4UART(slave_desc, soc_cfg, slave)
            hdl_obj.append(uart)
        elif slave_desc['type'] == 'acc_irq':
            irq_ctrl = Axi4Irq(slave_desc, soc_cfg, slave, mmap)
            hdl_obj.append(irq_ctrl)
        elif slave_desc['type'] == 'acc_timer':
            timer = Axi4Timer(slave_desc, soc_cfg, slave, mmap)
            hdl_obj.append(timer)
        elif slave_desc['type'] == 'acc_custom_slave':
            custom = Axi4AccCustomSlave(slave_desc, soc_cfg, slave, mmap)
            hdl_obj.append(custom)
        elif slave_desc['type'] == 'acc_rst':
            rst = Axi4RstCtrl(slave_desc, soc_cfg, slave, mmap)
            hdl_obj.append(rst)
        elif slave_desc['type'] == 'acc_noc':
            noc = RaveNoC(slave_desc, soc_cfg, slave, 'slave')
            hdl_obj.append(noc)
        elif slave_desc['type'] == 'acc_eth':
            eth_info['if'] += 1
            if slave_desc['eth_type'] == 'csr':
                eth_info['name'] = slave_desc['name']
                eth_info['clk_ext'] = slave_desc['clk_ext']
                eth_info['desc'] = slave_desc['desc']
                eth_info['csr_slv_id'] = slave
                eth_info['pkt_recv_irq'] = slave_desc['pkt_recv_irq']
                eth_info['pkt_sent_irq'] = slave_desc['pkt_sent_irq']
                eth_info['pkt_recv_full_irq'] = slave_desc['pkt_recv_full_irq']
            elif slave_desc['eth_type'] == 'infifo':
                eth_info['inf_slv_id'] = slave
            elif slave_desc['eth_type'] == 'outfifo':
                eth_info['out_slv_id'] = slave

            if eth_info['if'] == 3:
                eth = Axi4AccEthernet(soc_cfg, eth_info)
                hdl_obj.append(eth)
        else:
            logging.warning('Unknown slave - '+slave_desc['type'])
    return hdl_obj

def _soc_master(bus, soc, soc_cfg, dma_info):
    hdl_obj = []
    cpu_included = 0
    nox_cpu_included = 0
    vex_cpu_included = 0

    for master, master_desc in soc['masters'].items():
        if master_desc['type'] == 'cpu_nox':
            nox_cpu_included = 1
            if vex_cpu_included == 1:
                logging.error('Multiple CPUs included in master list!')
                sys.exit(1)
            if cpu_included == 0:
                cpu_included = 1
                hdl_obj.append(RISCVCpu(soc, bus.get_mmap(), 'nox'))
        elif master_desc['type'] == 'cpu_vex':
            vex_cpu_included = 1
            if nox_cpu_included == 1:
                logging.error('Multiple CPUs included in master list!')
                sys.exit(1)
            if cpu_included == 0:
                cpu_included = 1
                hdl_obj.append(RISCVCpu(soc, bus.get_mmap(), 'vex'))
        elif master_desc['type'] == 'acc_dma':
            dma_info[master_desc['name']] = master
            logging.debug('Master '+master_desc['type']+
                          ' will be added later in the slave side')
        elif master_desc['type'] == 'acc_custom_master':
            custom = Axi4AccCustomMaster(master_desc, soc_cfg, master)
            hdl_obj.append(custom)
        else:
            logging.warning('Unknown master type! - '+master_desc['type'])
    if cpu_included == 0 and soc['proc_required'] == 'y':
        logging.error('CPU not included in masters!')
        sys.exit(1)
    return hdl_obj

def _soc_io(hdl_obj):
    io_list = []
    input_signals = []
    output_signals = []
    for obj in hdl_obj:
        input_s = obj.get_io()['in']
        output_s = obj.get_io()['out']
        for i in input_s:
            if i not in input_signals:
                input_signals.append(i)
        for j in output_s:
            if j not in output_signals:
                output_signals.append(j)

    for i in range(len(input_signals)):
        tmp = 'input\t\t'+input_signals[i][0]+'\t'+input_signals[i][1]
        io_list.append(tmp)
    for i in range(len(output_signals)):
        tmp = 'output\t'+output_signals[i][0]+'\t'+output_signals[i][1]
        io_list.append(tmp)

    return io_list, input_signals, output_signals

def _gen_defines_mpsoc(noc, base_addr):
    defines = []

    noc_defines = {}
    noc_defines['flit_buff']            = noc['flit_buff']
    noc_defines['flit_data_width']      = noc['flit_data_width']
    noc_defines['h_priority']           = str(noc['h_priority'])
    noc_defines['routing_alg']          = str(noc['routing_alg'])
    noc_defines['max_sz_pkt']           = str(noc['max_sz_pkt'])
    noc_defines['noc_cfg_sz_rows']      = str(noc['size_x'])
    noc_defines['noc_cfg_sz_cols']      = str(noc['size_y'])
    noc_defines['ravenoc_base_addr']    = '\'h'+str(base_addr)
    noc_defines['n_virt_chn']           = str(noc['n_virt_chn'])
    noc_defines['rd_axi_bff(x)']        = 'x<=2?2:4'
    noc_defines['num_tiles']            = str(noc['size_x']*noc['size_y'])
    noc_defines['axi_wr_bff_base_addr'] = '`RAVENOC_BASE_ADDR+\'h1000'
    noc_defines['axi_rd_bff_base_addr'] = '`RAVENOC_BASE_ADDR+\'h2000'
    noc_defines['axi_csr_base_addr']    = '`RAVENOC_BASE_ADDR+\'h3000'
    noc_defines['axi_max_outstd_rd']    = 2
    noc_defines['axi_max_outstd_wr']    = 2

    defines.append(noc_defines)
    return defines



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File              : ipsocgen_cli.py
# License           : MIT license <Check LICENSE>
# Author            : Anderson Ignacio da Silva (aignacio) <anderson@aignacio.com>
# Date              : 06.02.2023
# Last Modified Date: 11.06.2023
import logging
import argparse
import pathlib
import sys
import yaml
import pkg_resources
try: # When released as a package
    from ipsocgen.common.constants import options, CustomFormatter
    from ipsocgen.common.validate import validate_config
    from ipsocgen.common.modules import *
    from ipsocgen.common.generate import *
    from ipsocgen.common.version import __IPSOCGEN_VERSION__
except ImportError: # While in development
    from common.constants import options, CustomFormatter
    from common.validate import validate_config
    from common.modules import *
    from common.generate import *
    from common.version import __IPSOCGEN_VERSION__

def _gen(cfg, output, mmap):
    if cfg['type'] == 'soc':
        soc_gen(cfg['soc_desc'], cfg['proj_name'], cfg['desc'], output, mmap, True)
    else:
        mpsoc_gen(cfg['mpsoc_desc'], cfg['proj_name'], cfg['desc'], output, mmap)

    return True

def main():
    version = __IPSOCGEN_VERSION__
    parser = argparse.ArgumentParser(description='IP SoC Generator CLI v'+version)

    parser.add_argument('-c','--cfg',
                        nargs='?',
                        type=argparse.FileType('r'),
                        help='YAML file with the configuration of the MP/SoC')

    parser.add_argument('-o','--output',
                        nargs='?',
                        type=pathlib.Path,
                        help='Output directory of the generated design',
                        default='./output')

    parser.add_argument('-v','--validate',
                        action='store_true',
                        help='Validate only configuration file')

    parser.add_argument('-d','--debug',
                        action='store_true',
                        help='Enable debug mode')

    parser.add_argument('-m','--mmap',
                        action='store_true',
                        help='Print memory map of each SoC')


    in_args = {}
    in_args['config']   = parser.parse_args().cfg
    in_args['output']   = parser.parse_args().output
    in_args['validate'] = parser.parse_args().validate
    in_args['debug']    = parser.parse_args().debug
    in_args['mmap']     = parser.parse_args().mmap

    handler = logging.StreamHandler()
    handler.setFormatter(CustomFormatter())
    logging.basicConfig(level=logging.DEBUG if in_args['debug'] == True else logging.INFO,
                        handlers=[handler])
    # print(in_args)
    if in_args['config'] == None:
            logging.warning("No valid configuration was specified, exiting now...")
            sys.exit(0)
    else:
        status, cfg = validate_config(in_args)
        if status == False:
            logging.error('Aborting generation...')
            sys.exit(1)
        else:
            if in_args['validate'] == True:
                logging.info('Validate only enabled, exiting now...')
                sys.exit(0)
            _gen(cfg, in_args['output'], in_args['mmap'])

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File              : constants.py
# License           : MIT license <Check LICENSE>
# Author            : Anderson Ignacio da Silva (aignacio) <anderson@aignacio.com>
# Date              : 06.02.2023
# Last Modified Date: 11.06.2023
import logging
import os
import abc
from tabulate import tabulate
from jinja2 import Environment, FileSystemLoader

class options:
    TYPES               = ['soc', 'mpsoc']
    sch_cfg             = os.path.join(os.path.dirname(__file__),'schemas/schema_config.yaml')
    sch_master          = os.path.join(os.path.dirname(__file__),'schemas/schema_master.yaml')
    sch_slave           = os.path.join(os.path.dirname(__file__),'schemas/schema_slave.yaml')
    tmpl_folder         = os.path.join(os.path.dirname(__file__),'templates')
    tmpl_axi4_cross_bar = 'axi4_crossbar.txt'
    tmpl_nox            = 'nox_wrapper.txt'
    tmpl_vex            = 'vex_wrapper.txt'
    tmpl_axi4_mem_ram   = 'axi4_mem_ram.txt'
    tmpl_axi4_mem_rom   = 'axi4_mem_rom.txt'
    tmpl_axi4_dma       = 'axi4_dma.txt'
    tmpl_axi4_uart      = 'axi4_uart.txt'
    tmpl_axi4_irq       = 'axi4_irq_ctrl.txt'
    tmpl_axi4_timer     = 'axi4_timer.txt'
    tmpl_axi4_custom_s  = 'axi4_custom_slave.txt'
    tmpl_axi4_custom_m  = 'axi4_custom_master.txt'
    tmpl_axi4_rst_ctrl  = 'axi4_rst_ctrl.txt'
    tmpl_axi4_ethernet  = 'axi4_ethernet.txt'
    tmpl_clk_pll_xlnx   = 'clk_pll_xilinx.txt'
    tmpl_clk            = 'clk_simple.txt'
    tmpl_rst            = 'rst_simple.txt'
    tmpl_module_header  = 'module_header.txt'
    tmpl_ravenoc        = 'ravenoc.txt'
    tmpl_module_inst    = 'module_instance.txt'
    tmpl_defines        = 'module_defines.txt'

    acc_addr_width              = {}
    acc_addr_width['acc_uart']  = 13
    acc_addr_width['acc_timer'] = 13
    acc_addr_width['acc_dma']   = 13
    acc_addr_width['acc_irq']   = 13
    acc_addr_width['acc_rst']   = 13
    acc_addr_width['acc_noc']   = 15
    acc_addr_width['acc_eth']   = 12
    allowed_size_kib_acc        = ('ram_mem', 'rom_mem', 'acc_custom_slave')

class CustomFormatter(logging.Formatter):
    white = "\x1b[97;20m"
    grey = "\x1b[38;20m"
    green = "\x1b[32;20m"
    cyan = "\x1b[36;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    fmt = "%(asctime)s - {}%(levelname)-8s{} - %(name)s.%(funcName)s - %(message)s"

    FORMATS = {
        logging.DEBUG: fmt.format(grey, reset),
        logging.INFO: fmt.format(green, reset),
        logging.WARNING: fmt.format(yellow, reset),
        logging.ERROR: fmt.format(red, reset),
        logging.CRITICAL: fmt.format(bold_red, reset),
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt, datefmt="%Y-%m-%d %H:%M:%S")
        return formatter.format(record)


class base_module:
    def __init__(self, name, clk, rst, tmpl):
        self.defines   = {}
        self.name      = name
        self.clk       = clk
        self.rst       = rst
        self.tmpl      = tmpl
        self.signals   = []
        self.io        = {}
        self.io['in']  = []
        self.io['out'] = []
        self.acc_type  = None

    def get_acc_type(self):
        return self.acc_type

    def get_io(self):
        return self.io

    def get_signals(self):
        return self.signals

    def get_params(self):
        params = []
        for attr_name, attr_value in self.__dict__.items():
            params.append([attr_name, attr_value])
        if logging.getLogger().isEnabledFor(logging.DEBUG):
            print(tabulate(params, tablefmt="double_outline"))

    def get_defines(self):
        return self.defines

    def get_hdl(self):
        file_loader = FileSystemLoader(options.tmpl_folder)
        env = Environment(loader=file_loader)
        template = env.get_template(self.tmpl)
        output = template.render(tmpl=self)
        # if logging.getLogger().isEnabledFor(logging.DEBUG):
            # print(output)
        return output



from setuptools import setup, find_packages
import codecs
import os
from ipsocgen.common.version import __IPSOCGEN_VERSION__

VERSION = __IPSOCGEN_VERSION__
DESCRIPTION = 'Generic SoC builder in HDL'
LONG_DESCRIPTION = 'Build SoCs (System-on-Chip) and MPSoCs (Multi-Processor) through yaml configuration files.'

here = os.path.abspath(os.path.dirname(__file__))
with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

# Setting up
setup(
    name="ipsocgen",
    version=VERSION,
    author="aignacio (Anderson Ignacio)",
    author_email="<anderson@aignacio.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    url='https://github.com/aignacio/ipsocgen',
    packages=find_packages(),
    package_data={
        '':['common/schemas/*.yaml', 'common/templates/*.txt']
    },
    include_package_data=True,
    install_requires=[
        'tabulate',
        'argparse',
        'pyyaml',
        'cerberus',
        'jinja2'
    ],
    keywords=['soc', 'mpsoc', 'hdl',  'verilog', 'systemverilog', 'builder'],
    entry_points = {
        'console_scripts': ['ipsocgen=ipsocgen.ipsocgen_cli:main'],
    },
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
